-- set serveroutput on;
-- --------------------------------------------------------------------------
--
-- Execute PL/SQL to create index :
--  - index foreign keys
--  - index on spatial columns (and registre column in SDO metadata)
--  - move index tablespace to correct valid tablespaces
--
--  project : SIH-Adagio :: Core Server
--  version : 4.1.40 for ${pom.env}
--      env : ${pom.env} - ${pom.profil}
--     date : ${pom.date.fr}
--
--  Copyright Ifremer 2009
-- 
-- 10/01/11 BL  Creation
-- 11/01/11 BL  Problème de perf en pre-prod V2 si on ne prend pas les FK qui 
--              sont dans une PK : donc on crée un index pour chaque FK (comme sur INST1)
-- 28/01/11 BL  Correction sur la création des index spatiaux, pour éviter d'en 
--              créer sur les view ou les tables de la corbeille (BIN$xxx)
-- 18/04/11 BL  Renommage des tablespaces en SIH2_ADAGIO_xxx
-- 17/05/11 BL  Mise en commentaire de la suppression des index sur FK, pour ne 
--              pas avoir à tous les recréer à chaque fois, mais seulement ceux qui manquent.
-- 06/07/11 BL  Ne plus utiliser de SRID dans les index spatiaux
-- --------------------------------------------------------------------------
-- --------------------------------------------------------------------------
-- IMPORTANT NOTE : Execute the sql file after connect as SIH2_ADAGIO_DBA
-- --------------------------------------------------------------------------


-- --------------------------------------------------------------------------
--
-- Register spatial tables in USER_SDO_GEOM_METADATA, 
-- then define spatial index on geometry column
--
-- --------------------------------------------------------------------------
DECLARE 
	-- define constantes :
	tablespace_name VARCHAR2(512) := 'SIH2_ADAGIO_SPA_INDEX';
	index_prefix VARCHAR2(512) := 'IX_SPA_';

	CURSOR cursorGeomIndexes is
	  select INDEX_NAME
	  from USER_INDEXES
	  where INDEX_NAME like index_prefix||'%';
	
	CURSOR cursorGeomTables is
	  select 
	    UTC.TABLE_NAME, 
	    UTC.COLUMN_NAME,
	    CASE
        WHEN UTC.TABLE_NAME = 'VESSEL_POSITION'  THEN 'POINT'
        WHEN UTC.TABLE_NAME = 'VESSEL_POSITION_VMS'  THEN 'POINT'
        WHEN UTC.TABLE_NAME = 'TRANSSHIPMENT'  THEN 'POINT'
        WHEN UTC.TABLE_NAME = 'OPERATION'  THEN 'MULTILINE'
        WHEN UTC.TABLE_NAME = 'FISHING_TRIP'  THEN 'MULTILINE'
	      WHEN INSTR(UTC.TABLE_NAME, '_AREA') > 0 THEN 'MULTIPOLYGON'
	      WHEN INSTR(UTC.TABLE_NAME, '_LINE') > 0 THEN 'MULTILINE'
	      WHEN INSTR(UTC.TABLE_NAME, '_POINT') > 0 THEN 'MULTIPOINT'
	    END GEOMETRY_TYPE
	  from USER_TAB_COLUMNS UTC, USER_TABLES UT
	  where UTC.DATA_TYPE = 'SDO_GEOMETRY'
	  and UTC.TABLE_NAME not like 'MD%T_%'
    and UTC.TABLE_NAME not like 'BIN$_%'
    and UTC.TABLE_NAME != 'PLAYGROUND_FISHING_EFFORT_ZONE'
    -- filtrer pour ne pas avoir les vues Oracle
    and UTC.TABLE_NAME = UT.TABLE_NAME;
	  
	sqlOrder VARCHAR2(512);
	indexName VARCHAR2(512);
	tableName VARCHAR2(512);
	columnName VARCHAR2(512);
	geometryType VARCHAR2(512);
	cur integer;
	rc  integer;
  
	geometryTypeNotFound EXCEPTION;
BEGIN 

  cur := DBMS_SQL.OPEN_CURSOR;
  
  -- Drop old indexes
  DBMS_OUTPUT.PUT_LINE('-- Dropping spatial indexes...');
  for cursorGeomIndex in cursorGeomIndexes loop
    indexName := cursorGeomIndex.INDEX_NAME;
    
    sqlOrder := '  Drop index '|| indexName;
    --DBMS_OUTPUT.PUT_LINE(sqlOrder);
    DBMS_SQL.PARSE(cur, sqlOrder, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Dropping spatial indexes [OK]');
        
  -- Unregistrer old sdo metadata
  DBMS_OUTPUT.PUT_LINE('-- Unregistering geometry metadata...');
  sqlOrder := '  Delete USER_SDO_GEOM_METADATA';
  --DBMS_OUTPUT.PUT_LINE(sqlOrder);
  DBMS_SQL.PARSE(cur, sqlOrder, DBMS_SQL.NATIVE);
  rc := DBMS_SQL.EXECUTE(cur);   
  DBMS_OUTPUT.PUT_LINE('-- Unregistering geometry metadata [OK]');

  -- registrer sdo metadata
  DBMS_OUTPUT.PUT_LINE('-- Registering geometry metadata...');
  for cursorGeomTable in cursorGeomTables loop
    tableName := cursorGeomTable.TABLE_NAME;
    columnName := cursorGeomTable.COLUMN_NAME;
    
    sqlOrder := '  Insert into USER_SDO_GEOM_METADATA '
              || '(TABLE_NAME, COLUMN_NAME, DIMINFO, SRID) '
              || 'values ( '
              || '    '''|| tableName||''',  '
              || '    '''|| columnName ||''', '
              || '    MDSYS.SDO_DIM_ARRAY( '
              || '		MDSYS.SDO_DIM_ELEMENT(''X'', -180, 180, .0005),  '
              || '		MDSYS.SDO_DIM_ELEMENT(''Y'', -90,90, .0005) '
              || '    ), '
              || '    null)'; -- SRID = null
    --DBMS_OUTPUT.PUT_LINE('  Register table/column '''|| tableName ||'/'|| columnName ||'''');
    DBMS_SQL.PARSE(cur, sqlOrder, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Registering geometry metadata [OK]');
		
  -- Create spatial indexes
  DBMS_OUTPUT.PUT_LINE('-- Creating spatial indexes...');
  for cursorGeomTable in cursorGeomTables loop
    tableName := cursorGeomTable.TABLE_NAME;
    columnName := cursorGeomTable.COLUMN_NAME;
    geometryType := cursorGeomTable.GEOMETRY_TYPE;

    if (geometryType is null) then
      RAISE geometryTypeNotFound;
    end if;
    sqlOrder := '  Create index '||index_prefix||substr(tableName,0,30-length(index_prefix))||' on '||tableName||' ( '
              || '  '|| columnName ||' '
              || ') '
              || 'indextype is mdsys.spatial_index '
			  || 'parameters (''tablespace='||tablespace_name||',layer_gtype='||geometryType||''')';
    --DBMS_OUTPUT.PUT_LINE(sqlOrder);
    DBMS_SQL.PARSE(cur, sqlOrder, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;        
  DBMS_OUTPUT.PUT_LINE('-- Creating spatial indexes [OK]');

  -- commit changes
  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
EXCEPTION
  WHEN geometryTypeNotFound THEN
    RAISE_APPLICATION_ERROR(-20001, 'Could not found type of the gemotry colunm (in table '||tableName||') to use for spatial index creation. Please update the cursor ''cursorGeomTables'' in the current PL/SQL.');
  WHEN OTHERS THEN
   RAISE;
END;
/

-- --------------------------------------------------------------------------
--
-- Create index on all foreign keys (except if already in primary key) 
-- 10/10/12 EB Exlusion des tables PERSON_SESSION%
-- --------------------------------------------------------------------------
DECLARE
  -- Selection des index sur les cles etrangeres
  CURSOR cursorIndex IS
    SELECT INDEX_NAME
    FROM USER_INDEXES
    WHERE USER_INDEXES.INDEX_NAME LIKE 'IX_FK_%';

  -- Selection des cles etrangeres sans index
  CURSOR cursorClesEtrangeres IS
    SELECT  
      USER_CONS_COLUMNS.TABLE_NAME, 
      USER_CONS_COLUMNS.COLUMN_NAME,
      'IX_FK_' || substr(USER_CONS_COLUMNS.TABLE_NAME, 0, 20)||'_' as INDEX_NAME,
      (case 
          when ut.tablespace_name='SIH2_ADAGIO_DATA' then 'SIH2_ADAGIO_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_REF_DATA' then 'SIH2_ADAGIO_REF_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_PLG_DATA' then 'SIH2_ADAGIO_PLG_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_SPA_DATA' then 'SIH2_ADAGIO_SPA_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_VES_DATA' then 'SIH2_ADAGIO_VES_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_TEC_DATA' then 'SIH2_ADAGIO_TEC_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_MES_DATA' then 'SIH2_ADAGIO_MES_INDEX'
          when ut.tablespace_name='SIH2_ADAGIO_VMS_DATA' then 'SIH2_ADAGIO_VMS_INDEX'
          when ut.tablespace_name='SIH2_ADAGIO_PRODUCE_DATA' then 'SIH2_ADAGIO_PRODUCE_INDEX'
          else 'SIH2_ADAGIO_WRK_DATA'
        end) as TABLESPACE_NAME
    FROM
      USER_CONSTRAINTS,
      USER_CONS_COLUMNS
        left outer join USER_IND_COLUMNS
          on USER_CONS_COLUMNS.TABLE_NAME = USER_IND_COLUMNS.TABLE_NAME
          and USER_CONS_COLUMNS.COLUMN_NAME = USER_IND_COLUMNS.COLUMN_NAME
          and USER_IND_COLUMNS.COLUMN_POSITION = 1
        left join user_tables ut
          on USER_CONS_COLUMNS.TABLE_NAME = ut.TABLE_NAME
    WHERE USER_CONSTRAINTS.TABLE_NAME = USER_CONS_COLUMNS.TABLE_NAME
      AND USER_CONSTRAINTS.CONSTRAINT_NAME = USER_CONS_COLUMNS.CONSTRAINT_NAME
      AND USER_CONSTRAINTS.CONSTRAINT_TYPE = 'R'
      -- BL : problème de perf en pre-prod V2 si on ne prend pas les FK qui sont dans une PK
      AND USER_IND_COLUMNS.TABLE_NAME is null
      AND USER_CONS_COLUMNS.TABLE_NAME != 'VESSEL_POSITION_VMS'
      -- EB : on ne prend pas en compte les tables partitionnées
      AND USER_CONS_COLUMNS.TABLE_NAME not like '%MEASUREMENT_P'
      -- EB : on ne prend pas en compte les tables techniques Allegro PERSON_SESSION
      AND USER_CONS_COLUMNS.TABLE_NAME not like 'PERSON_SESSION%'
      -- EB : on ne prend pas en compte certains champs des tables %MEASUREMENT
      AND
      (
       (
        USER_CONS_COLUMNS.TABLE_NAME like '%MEASUREMENT%'
        AND USER_CONS_COLUMNS.COLUMN_NAME not in (
        'AGGREGATION_LEVEL_FK',
        'PRECISION_TYPE_FK',
        'ANALYSIS_INSTRUMENT_FK',
        'NUMERICAL_PRECISION_FK',
        'DEPARTMENT_FK',
        'PMFM_FK',
        'QUALITY_FLAG_FK',
        'QUALITATIVE_VALUE_FK'
        ) 
        )
        or  USER_CONS_COLUMNS.TABLE_NAME not like '%MEASUREMENT%'
      )
     ORDER BY USER_CONS_COLUMNS.TABLE_NAME, USER_CONS_COLUMNS.COLUMN_NAME
      ;

  cur INTEGER;
  rc  integer;
  cpt INTEGER;

  indexName VARCHAR2(100);
  sqlQuery VARCHAR2(500);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;
  cpt := 1;

  -- Creation d'un index sur chaque cle etrangere
  DBMS_OUTPUT.PUT_LINE('-- Create indexes on all FK (with a prefixe ''IX_FK_'')...');
  FOR cleEtrangere IN cursorClesEtrangeres LOOP
    cpt := cpt + 1;

    sqlQuery := 'CREATE INDEX '||cleEtrangere.INDEX_NAME||cpt||' ON '||cleEtrangere.TABLE_NAME||'('||cleEtrangere.COLUMN_NAME||') TABLESPACE ' || cleEtrangere.TABLESPACE_NAME;
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);
  END LOOP;
  DBMS_OUTPUT.PUT_LINE('-- Create indexes on all FK (with a prefixe ''IX_FK_'') [OK]');

EXCEPTION
  WHEN OTHERS THEN
   RAISE;
END;
/


-- --------------------------------------------------------------------------
--
-- Change tablespace of spatial tables 
--
-- --------------------------------------------------------------------------
DECLARE 
	-- define constantes :
	tablespace_name VARCHAR2(512) := 'SIH2_ADAGIO_SPA_DATA';

	CURSOR cursorGeomTables is
	  select 
	    TABLE_NAME
	  from USER_TAB_COLUMNS
	  where DATA_TYPE = 'SDO_GEOMETRY'
	  and TABLE_NAME not like 'MD%T_%'
      -- except OPERATION and FISHING_TRIP tables
      and TABLE_NAME <> 'OPERATION'
      and TABLE_NAME <> 'FISHING_TRIP'
      and TABLE_NAME <> 'FISHING_TRIP_PUBLIC'
      and TABLE_NAME <> 'VESSEL_POSITION_VMS'
      and TABLE_NAME <> 'VESSEL_POSITION_COMPLETE';
	  
	sqlOrder VARCHAR2(512);
	tableName VARCHAR2(512);
	cur integer;
	rc  integer;
BEGIN 

  cur := DBMS_SQL.OPEN_CURSOR;
  
  -- registrer sdo metadata
  DBMS_OUTPUT.PUT_LINE('-- Changing tablespace of spatial tables...');
  for cursorGeomTable in cursorGeomTables loop
    tableName := cursorGeomTable.TABLE_NAME;
    
    sqlOrder := '  Alter table "'|| tableName||'" move tablespace "'||tablespace_name||'"';
    DBMS_OUTPUT.PUT_LINE(sqlOrder);
    DBMS_SQL.PARSE(cur, sqlOrder, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Changing tablespace of spatial tables [OK]');
  
  -- commit changes
  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/



-- --------------------------------------------------------------------------
--
-- Change INDEX tablespaces (only if need)
--
-- --------------------------------------------------------------------------
declare
	-- define constantes :
	spatial_index_prefix VARCHAR2(512) := 'IX_SPA_';

	CURSOR cursorIndexes is
		select * from (
      select
        UI.index_name as index_name, 
        UI.tablespace_name old_tablespace_name,
        (case 
          when ut.tablespace_name='SIH2_ADAGIO_DATA' then 'SIH2_ADAGIO_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_REF_DATA' then 'SIH2_ADAGIO_REF_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_PLG_DATA' then 'SIH2_ADAGIO_PLG_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_SPA_DATA' then 'SIH2_ADAGIO_SPA_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_VES_DATA' then 'SIH2_ADAGIO_VES_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_TEC_DATA' then 'SIH2_ADAGIO_TEC_INDEX' 
          when ut.tablespace_name='SIH2_ADAGIO_MES_DATA' then 'SIH2_ADAGIO_MES_INDEX'
          when ut.tablespace_name='SIH2_ADAGIO_VMS_DATA' then 'SIH2_ADAGIO_VMS_INDEX'
          when ut.tablespace_name='SIH2_ADAGIO_PRODUCE_DATA' then 'SIH2_ADAGIO_PRODUCE_INDEX'
          else 'SIH2_ADAGIO_WRK_DATA'
        end) as TABLESPACE_NAME      
      from 
        user_indexes UI,
        user_tables UT
      where 1=1 
        and UI.table_owner = USER
        and UI.table_name = UT.table_name
        and UI.index_type not in ('LOB','VARRAY', 'REF')
        and UI.index_name not like spatial_index_prefix||'%'
      order by UI.index_name
    ) where old_tablespace_name <> tablespace_name
    ;

	indexName VARCHAR2(512);
	tablespaceName VARCHAR2(512);
	cur integer;
	rc  integer;
	stmt varchar2(100);
	sqlQuery VARCHAR2(500);
begin

  DBMS_OUTPUT.PUT_LINE('-- Changing indexes tablespaces...');
  cur := DBMS_SQL.OPEN_CURSOR;
  for cursorIndex in cursorIndexes loop
      indexName := cursorIndex.index_name;
      tablespaceName := cursorIndex.TABLESPACE_NAME;
      
      sqlQuery := '  Alter index '||indexName||' rebuild tablespace '||tablespaceName;
      --DBMS_OUTPUT.PUT_LINE(sqlQuery);
      DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
      rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Changing indexes tablespaces [OK]');
  
  -- commit changes
  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
end;
/
