-- --------------------------------------------------------------------------
--
-- Execute PL/SQL to create triggers :
-- First, apply need 'alter table'
--  - Set UPDATE_DATE default value, as systimestamp.
--  - Set QUALITY_FLAG_FK default value to '0'
-- Then create triggers
--  - Add triggers to set to systimestamp when 'update_date' is null or not changed
--  - Add triggers to set to '0' when 'QUALITY_FLAG_FK' is null
--  - Create triggers for all ID column, using sequences (if a sequence exists)
--
--  project : SIH-Adagio :: Core Server
--  version : 4.1.40 for ${pom.env}
--      env : ${pom.env} - ${pom.profil}
--     date : ${pom.date.fr}
--
--  Copyright Ifremer 2009
--
-- 
-- 10/01/11 BL  Creation
-- 27/01/11 BL  Modification des trigger TR_%_UD pour ne pas s'executer 
--              sur le user SIH2_ADAGIO_BATCH_GEOMETRY (mise à jour des colonnes spatiales)
-- 17/05/11 BL  Modification de la procédure sur UPDATE_DATE pour prendre en compte
--              seulement les tables (et non plus les tables ET les vues)
-- 13/12/11 EM  Modification de la procédure sur QUALITY_FLAG et ID column pour prendre
--              en compte seulement les tables (et non plus les tables ET les vues) 
-- 14/12/11 BL  Surcharge des triggers T_SALE_UD et T_LANDING_UD generes, pour 
--              eviter la mise a jour de l'UPDATE_DATE si seulement la colonne
--              CATCH_BATCH_FK est modifi‚e (besoin pour la synchro Allegro V3)
-- 10/10/12 EB  Ajout du champ SAMPLING_STRATA_FK au trigger T_SALE_UD
-- 27/05/13 BL  Gestion des noms de trigger en évitant les cas de doublon de nom, 
--              qui empêchait la création de trigger sur certaines tables
--              (comme QUANTIFICATION_MEASUREMENT_P et QUANTIFICATION_MEASUREMENT_B, car nom trop long)
--              Creation d'un trigger TR_QUANTIFICATION_MEAS_P_ID qui n'est pas créé pour le PL/SQL générique
-- --------------------------------------------------------------------------
-- --------------------------------------------------------------------------
-- IMPORTANT NOTE : Execute the sql file after connect as SIH2_ADAGIO_DBA
-- --------------------------------------------------------------------------
--SET SERVEROUTPUT ON

-- --------------------------------------------------------------------------
--
-- Set UPDATE_DATE default value, as systimestamp. (Only if need)
--
-- --------------------------------------------------------------------------
DECLARE  
  CURSOR cursorTableNamesNoDefaultValue is
    select 
      UTC.TABLE_NAME, 
      'TR_' || substr(UTC.TABLE_NAME, 0, 24)||'_UD' AS TRIGGER_NAME
    from 
      USER_TAB_COLUMNS UTC,
      USER_TABLES UT
    where 
      UTC.column_name = 'UPDATE_DATE'
      and UTC.data_default is null
      and UTC.TABLE_NAME not like 'MD%T_%'
      and UTC.TABLE_NAME not like 'BIN$%'
      and UTC.TABLE_NAME = UT.TABLE_NAME;
  CURSOR cursorTableNamesNullable is
	  select 
	    UTC.TABLE_NAME
	  from
      USER_TAB_COLUMNS UTC,
      USER_TABLES UT
	  where 
	    UTC.column_name = 'UPDATE_DATE'
	    and UTC.TABLE_NAME not like 'MD%T_%'
	    and UTC.TABLE_NAME not like 'BIN$%'
		  and UTC.NULLABLE ='Y'
      and UTC.TABLE_NAME = UT.TABLE_NAME;
  cur integer;
  rc  integer;
  triggerName VARCHAR2(500);
  tableName VARCHAR2(500);
  sqlQuery VARCHAR2(500);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;

  DBMS_OUTPUT.PUT_LINE('-- Setting update_date into NOT NULL...');
  for cursorTableName in cursorTableNamesNullable loop
    
    tableName := cursorTableName.TABLE_NAME;
    DBMS_OUTPUT.PUT_LINE('Alter table '''|| tableName ||'''');
    
    sqlQuery := '  update '||tableName
             ||' set UPDATE_DATE = systimestamp'
             ||' where UPDATE_DATE is null';  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   

    sqlQuery := '  ALTER TABLE '||tableName
             ||' MODIFY (UPDATE_DATE NOT NULL)';
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Setting update_date into NOT NULL [OK]');

  DBMS_OUTPUT.PUT_LINE('-- Changing default value to ''systimestamp''...');
  for cursorTableName in cursorTableNamesNoDefaultValue loop
    
    tableName := cursorTableName.TABLE_NAME;
    DBMS_OUTPUT.PUT_LINE('Alter table '''|| tableName ||'''');
    
    sqlQuery := '  ALTER TABLE '||tableName
             ||' MODIFY (UPDATE_DATE DEFAULT systimestamp)';
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Changing default value [OK]');

  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/


-- --------------------------------------------------------------------------
--
-- Set QUALITY_FLAG_FK default value, as '0' (not qualified).
--
-- --------------------------------------------------------------------------
DECLARE
  CURSOR cursorTableNamesNoDefaultValue is
    select 
      UTC.TABLE_NAME, 
      'TR_' || substr(UTC.TABLE_NAME, 0, 24)||'_QF' AS TRIGGER_NAME
    from
      USER_TAB_COLUMNS UTC,
      USER_TABLES UT
    where 
      UTC.column_name = 'QUALITY_FLAG_FK'
      and UTC.data_default is null
      and UTC.TABLE_NAME not like 'MD%T_%'
      and UTC.TABLE_NAME not like 'BIN$%'
      and UTC.TABLE_NAME = UT.TABLE_NAME;
   CURSOR cursorTableNamesNullable is
      select 
        UTC.TABLE_NAME
      from
        USER_TAB_COLUMNS UTC,
        USER_TABLES UT
      where 
        UTC.column_name = 'QUALITY_FLAG_FK'
        and UTC.TABLE_NAME not like 'MD%T_%'
        and UTC.TABLE_NAME not like 'BIN$%'
        and UTC.NULLABLE ='Y'
        and UTC.TABLE_NAME = UT.TABLE_NAME;
  cur integer;
  rc  integer;
  defaultQualityFlag VARCHAR2(40);
  triggerName VARCHAR2(500);
  tableName VARCHAR2(500);
  sqlQuery VARCHAR2(500);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;

  DBMS_OUTPUT.PUT_LINE('-- Setting QUALITY_FLAG_FK into NOT NULL...');
  for cursorTableName in cursorTableNamesNullable loop
    
    tableName := cursorTableName.TABLE_NAME;
    DBMS_OUTPUT.PUT_LINE('Alter table '''|| tableName ||'''');
    
    sqlQuery := '  update '||tableName
             ||' set QUALITY_FLAG_FK = ''0'''
             ||' where QUALITY_FLAG_FK is null';  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   

    sqlQuery := '  ALTER TABLE '||tableName
             ||' MODIFY (QUALITY_FLAG_FK NOT NULL)';
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Setting QUALITY_FLAG_FK into NOT NULL [OK]');

  DBMS_OUTPUT.PUT_LINE('-- Changing default value to ''0''...');
  for cursorTableName in cursorTableNamesNoDefaultValue loop
    
    tableName := cursorTableName.TABLE_NAME;
    DBMS_OUTPUT.PUT_LINE('Alter table '''|| tableName ||'''');
    
    sqlQuery := '  ALTER TABLE '||tableName
             ||' MODIFY (QUALITY_FLAG_FK DEFAULT ''0'')';
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Changing default value [OK]');

  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/

-- --------------------------------------------------------------------------
--
-- Add trigger for insert and update operations, for 'update_date' column
-- -> set to systimestamp when 'update_date' is null or not changed
--
-- --------------------------------------------------------------------------
DECLARE
  CURSOR cursorTriggerNames is
    select TRIGGER_NAME 
    from USER_TRIGGERS
    where 
      TRIGGER_NAME like 'TR_%_UD';
  CURSOR cursorTableNames is
    select 
      UTC.TABLE_NAME, 
      'TR_' || substr(UTC.TABLE_NAME, 0, 24)||'_UD' AS TRIGGER_NAME,
      case 
        when UTC.TABLE_NAME in ('OPERATION', 'FISHING_TRIP', 'VESSEL_POSITION') then 1 
        else 0
      end  as IS_GEOMETRY_TABLE
    from USER_TAB_COLUMNS UTC
    left outer join USER_VIEWS UV
      on UTC.TABLE_NAME = UV.VIEW_NAME
    where 
      UTC.column_name = 'UPDATE_DATE'
      and UTC.TABLE_NAME not like 'MD%T_%'
      and UTC.TABLE_NAME not like 'BIN$%'
      and UV.VIEW_NAME is null;
  cur integer;
  rc  integer;
  triggerName VARCHAR2(500);
  tableName VARCHAR2(500);
  sqlQuery VARCHAR2(500);
  alternativeTriggerName VARCHAR2(30);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;

  DBMS_OUTPUT.PUT_LINE('-- Deleting old update_date triggers...');
  for cursorTriggerName in cursorTriggerNames loop
    triggerName := cursorTriggerName.TRIGGER_NAME;
    sqlQuery := '  drop trigger '||triggerName;
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Deleting old update_date triggers [OK]');
  
  DBMS_OUTPUT.PUT_LINE('-- Creating triggers...');
  for cursorTableName in cursorTableNames loop
    
    tableName := cursorTableName.TABLE_NAME;
    triggerName := cursorTableName.TRIGGER_NAME;
    
    -- Check if triggerName is unique : if not, compute a new trigegr name (delete one character)
    BEGIN
      select
        substr(trigger_name, 0, LENGTH(trigger_name) - 4 ) || substr(trigger_name, LENGTH(trigger_name) - 2)
        into alternativeTriggerName
      from user_triggers
      where trigger_name = triggerName;
      triggerName := alternativeTriggerName;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
          triggerName := triggerName;
    END;
    
    DBMS_OUTPUT.PUT_LINE('Create trigger '''|| triggerName ||'''');
    
    sqlQuery := '  Create or replace trigger '||triggerName
             ||' before insert or update on  '||tableName
             ||' for each row';
             -- special case if table with a geometry column : do not update UD if used the special user SIH2_ADAGIO_BATCH_GEOMETRY
    if (cursorTableName.IS_GEOMETRY_TABLE = 1) then
      sqlQuery := sqlQuery || ' when (((new.UPDATE_DATE is null) or (new.UPDATE_DATE = old.UPDATE_DATE)) and (USER <> ''SIH2_ADAGIO_BATCH_GEOMETRY''))';
    else
      sqlQuery := sqlQuery || ' when ((new.UPDATE_DATE is null) or (new.UPDATE_DATE = old.UPDATE_DATE))';
    end if;
    sqlQuery := sqlQuery || ' begin'
             ||'    :new.UPDATE_DATE := systimestamp;'
             ||' end; ';
             
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   

  end loop;
  DBMS_OUTPUT.PUT_LINE('Creating triggers [OK]');

  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/


-- --------------------------------------------------------------------------
--
-- Add trigger for insert and update operations, for 'QUALITY_FLAG_FK' column
-- -> set to '0' when 'QUALITY_FLAG_FK' is null
--
-- --------------------------------------------------------------------------
DECLARE
  CURSOR cursorTriggerNames is
    select TRIGGER_NAME 
    from USER_TRIGGERS
    where 
      TRIGGER_NAME like 'TR_%_QF';
  CURSOR cursorTableNames is
    select 
      UTC.TABLE_NAME, 
      'TR_' || substr(UTC.TABLE_NAME, 0, 24)||'_QF' AS TRIGGER_NAME
	from USER_TAB_COLUMNS UTC
	left outer join USER_VIEWS UV
      on UTC.TABLE_NAME = UV.VIEW_NAME
    where 1=1
      and UTC.TABLE_NAME not like 'MD%T_%'
      and UTC.TABLE_NAME not like 'BIN$%'
      and UV.VIEW_NAME is null
      and UTC.column_name = 'QUALITY_FLAG_FK';
  cur integer;
  rc  integer;
  defaultQualityFlag VARCHAR2(40);
  triggerName VARCHAR2(500);
  tableName VARCHAR2(500);
  sqlQuery VARCHAR2(500);
  alternativeTriggerName VARCHAR2(30);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;

  DBMS_OUTPUT.PUT_LINE('-- Deleting old update_date triggers...');
  for cursorTriggerName in cursorTriggerNames loop
    triggerName := cursorTriggerName.TRIGGER_NAME;
    sqlQuery := '  drop trigger '||triggerName;
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Deleting old update_date triggers [OK]');
  
  DBMS_OUTPUT.PUT_LINE('-- Creating triggers...');
  for cursorTableName in cursorTableNames loop
    
    tableName := cursorTableName.TABLE_NAME;
    triggerName := cursorTableName.TRIGGER_NAME;
   
    -- Check if triggerName is unique : if not, compute a new trigegr name (delete one character)
    BEGIN
      select
        substr(trigger_name, 0, LENGTH(trigger_name) - 4 ) || substr(trigger_name, LENGTH(trigger_name) - 2)
        into alternativeTriggerName
      from user_triggers
      where trigger_name = triggerName;
      triggerName := alternativeTriggerName;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
          triggerName := triggerName;
    END;    
    
    DBMS_OUTPUT.PUT_LINE('Create trigger '''|| triggerName ||'''');
    
    sqlQuery := '  Create or replace trigger '||triggerName
             ||' before insert or update on  '||tableName
             ||' for each row'
             ||' when (new.QUALITY_FLAG_FK is null)'
             ||' begin'
             ||'    :new.QUALITY_FLAG_FK := ''0'';'
             ||' end; ';
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('Creating triggers [OK]');

  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/


-- --------------------------------------------------------------------------
--
-- Create triggers for all ID column, using sequences (if a sequence exists) 
--
-- --------------------------------------------------------------------------
DECLARE
    CURSOR cursorTriggerNames is
    select 
      TRIGGER_NAME 
    from 
      USER_TRIGGERS
    where 
      TRIGGER_NAME like 'TR_%_ID';
	CURSOR cursorSequences is
    select 
      us.sequence_name,
      ut.table_name,
      uc.column_name,
      'TR_'||substr(ut.table_name, 0, 26-length(uc.column_name))||'_ID' as trigger_name
    from 
      user_sequences us,
      user_tables ut,
      USER_TAB_COLUMNS uc
	left outer join USER_VIEWS UV
      on UC.TABLE_NAME = UV.VIEW_NAME
    where 1=1
     and us.sequence_name like '%_SEQ'
     and us.sequence_name = substr(ut.table_name,0,26)||'_SEQ'
     and uc.table_name = ut.table_name
     and uc.column_name = 'ID'
	 and uv.view_name is null
    order by uc.table_name;

	cur integer;
	rc  integer;  
	triggerName VARCHAR2(500);
	tableName VARCHAR2(500);
	sequenceName VARCHAR2(500);
	columnName VARCHAR2(500);  
	sqlQuery VARCHAR2(500);
	alternativeTriggerName VARCHAR2(30);
BEGIN
  cur := DBMS_SQL.OPEN_CURSOR;

  DBMS_OUTPUT.PUT_LINE('-- Deleting old ID triggers...');
  for cursorTriggerName in cursorTriggerNames loop
    triggerName := cursorTriggerName.TRIGGER_NAME;
    sqlQuery := '  drop trigger '||triggerName;
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  DBMS_OUTPUT.PUT_LINE('-- Deleting old ID triggers [OK]');

  DBMS_OUTPUT.PUT_LINE('-- Adding trigger to generate ID from sequence...');
  for cursorSequence in cursorSequences loop
    
    tableName := cursorSequence.TABLE_NAME;
    sequenceName := cursorSequence.SEQUENCE_NAME;
    columnName := cursorSequence.COLUMN_NAME;
    triggerName := cursorSequence.TRIGGER_NAME;
    
    -- Check if triggerName is unique : if not, compute a new trigegr name (delete one character)
    BEGIN
      select
        substr(trigger_name, 0, LENGTH(trigger_name) - 4 ) || substr(trigger_name, LENGTH(trigger_name) - 2)
        into alternativeTriggerName
      from user_triggers
      where trigger_name = triggerName;
      triggerName := alternativeTriggerName;
    EXCEPTION
      WHEN NO_DATA_FOUND THEN
          triggerName := triggerName;
    END;    

   sqlQuery := '  Create or replace trigger '||triggerName
            || '   before insert on '||tableName
            || '   for each row'
            || ' begin'
            || '  if (:new.'||columnName||' is null) then'
            || '   select '||sequenceName||'.nextval'
            || '     into :new.'||columnName
            || '     from dual;'
            || '  end if;'
            || ' end;';
    --DBMS_OUTPUT.PUT_LINE('  Create trigger '''||triggerName||'''');
    DBMS_OUTPUT.PUT_LINE(sqlQuery);
  
    DBMS_SQL.PARSE(cur, sqlQuery, DBMS_SQL.NATIVE);
    rc := DBMS_SQL.EXECUTE(cur);   
  end loop;
  
  DBMS_OUTPUT.PUT_LINE('-- Adding trigger to generate ID from sequence [OK]');

  -- commit changes
  COMMIT;
  DBMS_SQL.CLOSE_CURSOR(cur);
END;
/


-- --------------------------------------------------------------------------
-- 14/12/11 BL  Surcharge du trigger T_SALE_UD genere pour eviter la mise a jour de l'UPDATE_DATE
--              si seulement la colonne CATCH_BATCH_FK est modifi‚e (besoin pour la synchro Allegro V3)
-- 10/10/12 EB  Ajout du champ SAMPLING_STRATA_FK
-- 29/08/12 EB retrait du champ SAMPLING_PLAN_REFERENCE
-- --------------------------------------------------------------------------
CREATE OR REPLACE TRIGGER "TR_SALE_UD"
before INSERT OR UPDATE OF
  ID,SALE_START_DATE,CONTROL_DATE,VALIDATION_DATE,QUALIFICATION_DATE,QUALIFICATION_COMMENTS,DECLARED_DOCUMENT_REFERENCE_FK,FISHING_TRIP_FK,SALE_LOCATION_FK,SALE_TYPE_FK,VESSEL_FK,QUALITY_FLAG_FK,IS_OBSERVED,SALE_END_DATE, COMMENTS, CREATION_DATE, UPDATE_DATE,
  LANDING_FK,PROGRAM_FK,RECORDER_DEPARTMENT_FK,RECORDER_PERSON_FK,SELLER_FK,TAKE_OVER_FK,SAMPLING_STRATA_FK                                                                                                                                                                                  
ON SALE 
FOR EACH row 
  WHEN ((new.UPDATE_DATE IS NULL)
        OR (new.UPDATE_DATE = old.UPDATE_DATE))
BEGIN
  :new.UPDATE_DATE := systimestamp;
END;        
/
-- --------------------------------------------------------------------------
-- 14/12/11 BL  Surcharge du trigger TR_LANDING_UD genere pour eviter la mise a jour de l'UPDATE_DATE
--              si seulement la colonne CATCH_BATCH_FK est modifi‚e (besoin pour la synchro Allegro V3)
-- --------------------------------------------------------------------------
create or replace
trigger TR_LANDING_UD
before insert or update
of
ID,IS_OBSERVED,LANDING_DATE_TIME,COMMENTS,CREATION_DATE,CONTROL_DATE,VALIDATION_DATE,QUALIFICATION_DATE,QUALIFICATION_COMMENTS,UPDATE_DATE,
VESSEL_FK,PROGRAM_FK,RECORDER_DEPARTMENT_FK,RECORDER_PERSON_FK,QUALITY_FLAG_FK,FISHING_TRIP_FK,LANDING_LOCATION_FK,RANK_ORDER
on LANDING
for each row when ((new.UPDATE_DATE is null) or (new.UPDATE_DATE = old.UPDATE_DATE))
begin
  :new.UPDATE_DATE := systimestamp;
end; 
/

-- --------------------------------------------------------------------------
-- 27/05/13 BL  Creation d'un trigger TR_QUANTIFICATION_MEAS_P_ID qui n'est pas créé pour le PL/SQL générique
-- --------------------------------------------------------------------------
create or replace
trigger TR_QUANTIFICATION_MEAS_P_ID   before insert on QUANTIFICATION_MEASUREMENT_P   for each row begin  if (:new.ID is null) then   select QUANTIFICATION_MEASUREMENT_SEQ.nextval     into :new.ID     from dual;  end if; end;